SearchAmicableNumbers:=function(n)
#
# Amicable numbers are two different numbers such that the sum 
# of the proper divisors of one of the numbers is equal to the other
#
local l,i,j; # locals
l:=[];
for i in [ 1 .. n-1 ] do
  Print( i, "\r");
  for j in [i+1.. n ] do
    if (Sum(DivisorsInt(j))=i+j) and
       (Sum(DivisorsInt(i))=i+j) then
      Add (l,[i,j]);
      Print ([i,j], " - amicable!\n");
    fi;
  od;
od;
return l;
end; 

SearchAmicableNumbers(300);

SearchPerfectNumbers:=function(n1,n2)
#
# This function returns a list of all perfect numbers 
# from [ n1 .. n2 ]. Remind that a natural number k is 
# said to be perfect if it is equal to the sum of all
# its proper divisors, e.g. 6=1+2+3.
#
local i, l;
l:=[];
for i in [ n1 .. n2 ] do
    Print(i, " \r");  
    if Sum(DivisorsInt(i))-i=i then
       Print(i, " - perfect! \n");
       Add(l,i);          
    fi;
od;
return l;
end;
SearchPerfectNumbers(1,10000);

########################################################################
#
# Group Libraries
#
# The function searches in the GAP Small Groups Library for a group of
# the smallest order |G| where 1 <= G <= n such that the average order 
# of its elements is an integer.
#
# School challenge: improve the algorithm and find more groups with such 
# property (requires acquaintance with the basics of the group theory)
#
Test:=function(n)
local s, k, i, G, sum, c;
for s in [ 1 .. n ] do
  k := NrSmallGroups(s);
  Print("Analyzing ", k, " groups of order ", s, "\n");
  for i in [ 1 .. k ] do
    Print(i,"\r");
    G := SmallGroup(s,i);
    if not IsAbelian( G ) then 
      sum := 0;
      for c in ConjugacyClasses( G ) do
        sum := sum + Size( c ) * Order( Representative( c ) );
      od;
      sum := sum/Size(G);
      if IsInt( sum ) then
          Print (IdGroup(G), " has an average order of element equal to ", sum, "\n");                          
          return G;
      fi;  
    fi;
  od;  
od;
end;
Test(200);
#
# Rubik's cube
#
#                 +--------------+
#                 |  1    2    3 |
#                 |  4  top    5 |
#                 |  6    7    8 |
#  +--------------+--------------+--------------+--------------+
#  |  9   10   11 | 17   18   19 | 25   26   27 | 33   34   35 |
#  | 12  left  13 | 20 front  21 | 28 right  29 | 36  back  37 |
#  | 14   15   16 | 22   23   24 | 30   31   32 | 38   39   40 |
#  +--------------+--------------+--------------+--------------+
#                 | 41   42   43 |
#                 | 44  down  45 |
#                 | 46   47   48 |
#                 +--------------+
#
cube := Group(
  ( 1, 3, 8, 6)( 2, 5, 7, 4)( 9,33,25,17)(10,34,26,18)(11,35,27,19),
  ( 9,11,16,14)(10,13,15,12)( 1,17,41,40)( 4,20,44,37)( 6,22,46,35),
  (17,19,24,22)(18,21,23,20)( 6,25,43,16)( 7,28,42,13)( 8,30,41,11),
  (25,27,32,30)(26,29,31,28)( 3,38,43,19)( 5,36,45,21)( 8,33,48,24),
  (33,35,40,38)(34,37,39,36)( 3, 9,46,32)( 2,12,47,29)( 1,14,48,27),
  (41,43,48,46)(42,45,47,44)(14,22,30,38)(15,23,31,39)(16,24,32,40));
Size(cube);
f:=FreeGroup("T","L","F","R","B","D");
hom:=GroupHomomorphismByImagesNC( f, 
                                  cube,
                                  GeneratorsOfGroup( f ),
                                  GeneratorsOfGroup( cube ) );
count:=0;
sum:=0;
for i in [ 1 .. 100] do
  Print("************************************************************\n\n");
  count:=count+1;
  Print("Test # ", count, "\n\n");
  x := Random( cube );
  Print("Initial state of the cube : \n", x, "\n\n");
  w := PreImagesRepresentative( hom, x );
  Print("Solution of the cube : \n", w, "\n\n"); 
  Print("Number of moves : ", Length(w), "\n\n");
  sum:=sum+Length(w);
  Print("Average number of moves : ", Int(sum/count), "\n\n");
od;
#
# HOW TO FIND YOUR WAY IN THE GAP LIBRARY ???
#
G:=SymmetricGroup(7);      
KnownAttributesOfObject(G);
Centre(G);
time;
KnownAttributesOfObject(G);
Centre(G);
time;
Print( ApplicableMethod(Centre,[G]),"\n");      
G:=SymmetricGroup(7);      
Print( ApplicableMethod(Centre,[G]),"\n");       
G:=SmallGroup(256,42);
Print( ApplicableMethod(Centre,[G]),"\n"); 
#
# ADDING NEW METHODS
#
DeclareAttribute( "AverageOrder", IsGroup );

InstallMethod( AverageOrder, 
               "for good groups", 
               true, 
               [IsGroup], 
               0, 
    function( G )
    local sum, c;
    sum := 0;
    for c in ConjugacyClasses( G ) do
        sum := sum + Size( c ) * Order( Representative( c ) );
    od;
    return sum/Size(G);
    end);

G := SymmetricGroup( 20 );
AverageOrder( G );
time;
AverageOrder( G );
time;